/* Version 2012-06
 * In this version, each individual has a genome consisting of
 * a number of 24 bit genes represented by unsigned long integers.
 * The strength of the individual is the value of that integer.
 * At present, the genome has two genes.
 * Initially, each gene is a random sequence of ones and zeroes.
 * Each time, before producing seeds, the individual has to determine
 * if it should mutate or not.
 *
 * A mutation turns a 0 bit to 1 and a 1 bit to 0
 *
 */

#include "allan.cpp"   // Functions and data for handling routine tasks
// Random number generator by Makoto Matsumoto and Takuji Nishimura
// has been installed

#include "darwin.h"      // Declarations and definitions
#include "objects.cpp"   // Active components, like individuals and world
#include "display.cpp"   // Functions for presenting the result, like Draw
                         // that displays distribution of individuals

void Catastrophe()
{
 int i,
     n = 0,
     x,
     x_survive,
     y,
     y_survive;

 // All individual outside a sub square will perish
 // Side of sub square is 13 % of total world square
 // Decide position in x direction of surviving square
 x_survive = 0.85 * Side * Random_float();
 y_survive = 0.85 * Side * Random_float();

 for( x = 0; x < Side; x++ )
  for( y = 0; y < Side; y++ )
   if( x >= x_survive && x < x_survive + 0.13 * Side &&
       y >= y_survive && y < y_survive + 0.13 * Side )
   {
    if( World[ x ][ y ].plant > NONE )
    {
     Memory[ n ] = Individual[ World[ x ][ y ].plant ].copy();
     n++;
    }
   }
   else
    World[ x ][ y ].plant = NONE;   // Dead

 // Rearrange the population. Keep the survivals only
 for( i = 0; i < n; i++ )
  Individual[ i ] = Memory[ i ].copy();
 Population = n;
 Draw();
}

void Create()    // Create environment and life
{
 int bit,
     gene,
     g,
     size,
     x,
     y;

 size = StrToInt( Form1 -> Edit3  -> Text );  // Get size of the world (no of locations)
 Side = sqrt( (float)size );
 if( Side > MAX ) Side = MAX;                 // Make sure that illegal number is corrected

 Mutationprobability = StrToFloat( Form1 -> Edit2  -> Text );

 Last_random = StrToInt( Form1 -> Edit4  -> Text );
 Start_random_gen( Last_random );

 // Remove all possible life
 for( x = 0; x < Side; x++ )
  for( y = 0; y < Side; y++ )
  {
   World[ x ][ y ].winter();
   World[ x ][ y ].kill_seeds();
  }

 Coordinates( -1, -1, Side + 1, Side + 1 );

 Max_gene_sum = GENES * MAXGENE;

 // Create first life
 for( g = FIRST; g < GENES; g++ )
 {
  Individual[ FIRST ].gene[ g ] = EMPTY;    // Prepare the gene to be filled
  for( bit = FIRST; bit < GENESIZE - 2; bit++ )
  {
   if( Random_float() > 0.5 ) // then switch the corresponding bit to one
    Individual[ FIRST ].gene[ g ] = Individual[ FIRST ].gene[ g ] | Mutationmask[ bit ];
  }
  // Set the two most significant bits yo one. Otherwise, lethal.
  Individual[ FIRST ].gene[ g ] = Individual[ FIRST ].gene[ g ] | Mutationmask[ GENESIZE - 2 ];
  Individual[ FIRST ].gene[ g ] = Individual[ FIRST ].gene[ g ] | Mutationmask[ GENESIZE - 1 ];
 }
 Individual[ FIRST ].find_strength();        // Find how strong it is
 Individual[ FIRST ].find_species();         // Find what species it belongs to
 Individual[ FIRST ].x = Side / 2;           // Make it know its own position
 Individual[ FIRST ].y = Side / 2;

 // Put the first life into the world
 World[ Side / 2 ][ Side / 2 ].plant = FIRST; // Put an individual at centre
 Population = 1;  // Number of individuals initially in the world
 Total_generations = 1;
 Fitness = Individual[ FIRST ].strength;  // Fitness of largest population
 Mutations = 0;
 Max_strength = Individual[ FIRST ].strength;
 Form1 -> Edit4 ->  Text    = Fitness * 100;
 Form1 -> Edit5 ->  Text    = Find_favor();
 Draw();
}

// Find which seed will grow to make a plant
void Fight( int x, int y ) // x and y are positions in the grid
{
 float sum1 = 0,  // Sum of fitness parameters for all seeds in the position
       sum2 = 0;  // Sum to decide wich seed to choose

 int chosen_seed,
     position;

 float r;

 // First make a sum of all fitness parameters of seeds in the position
 for( position = 0; position < 5; position++ )
 {
  if( World[ x ][ y ].seed[ position ] > NONE )
   sum1 += Memory[ World[ x ][ y ].seed[ position ] ].strength;
 }
 if( sum1 > 0 ) //Are there any seeds at all?
 {
  r=Random_float();
  sum1 *= r;

  for( position = 0; position < 5; position++ )
  {
   //Take one seed at a time
   if( World[ x ][ y ].seed[ position ] > NONE )
    sum2 += Memory[ World[ x ][ y ].seed[ position ] ].strength;
   // Check if the present seed is chosen
   // If so, the random sum should be in the interval covered by the seed
   if( sum2 > sum1 ) break;
  }
  chosen_seed = World[ x ][ y ].seed[ position ];
  // Let the chosen seed grow to a full size plant
  Individual[ Population ].grow( Memory[ chosen_seed ], x, y );

  // Kill the seeds
  World[ x ][ y ].kill_seeds();

  World[ x ][ y ].plant = Population;
  Population++;
 }
 else
 {
  World[ x ][ y ].plant = NONE; // No seeds there
 }
}

float Find_favor()  //  Find probability for favourable mutation in entire population
{
 int bit,
     g,
     ones = 0,
     x,
     y,
     zeros;

 for( x = 0; x < Side; x++ )
  for( y = 0; y < Side; y++ )
  {
   if( World[ x ][ y ].plant > NONE )
   {
    for( g = 0; g < GENES; g++ )            // Check all genes of the individual
    {
     for( bit = 0; bit < GENESIZE; bit++ )  // Check each bit in the gene
     {
      if( ( Individual[ World[ x ][ y ].plant ].gene[ g ] & Mutationmask[ bit ] ) > 0 )
       ones++;
     }
    }
   }
  }
 zeros = GENESIZE * GENES * Population - ones;
 return( (float)(zeros * 100 / (float)Population) / ( float )( GENESIZE * GENES ) );
}

void Generations()
{
 int displaycounter,
     i;

 // Read the number of generations to process
 Duration = StrToInt( Form1 -> Edit1  -> Text );
 Mutationprobability = StrToFloat( Form1 -> Edit2  -> Text );

// Future extension:
// if( Genes > 1 )
//  if( Form1 -> Edit1  -> CheckBox1 == cbChecked = Gene_coupling = true;

 for( i = 0; i < Duration; i++ )
 {
  One_generation();
  if( displaycounter++ > 9 )
  {
   Draw();
   displaycounter = 0;
  }
 }
 Speciation();
 Form1 -> Edit1  -> Text = Duration;
 Form1 -> Edit4 ->  Text    = Fitness * 100;
 Form1 -> Edit5 ->  Text    = Find_favor();
 Form1 -> Edit6 ->  Text    = Total_generations;
 Form1 -> Edit7 ->  Text    = Mutations;
 Draw();
}

void One_generation()
{
 int i,
     x,
     y;

 // Let each plant spread its seeds to closest locations
 for( i = 0; i < Population; i++ )
 {
  Individual[ i ].spread_seeds( i );
  Memory[ i ] = Individual[ i ].copy();  // Remember individuals from the old generation
 }                                       // To increase speed, each seed does not carry full
 Old_population = Population;            // genetic information. When growing, it will read
                                         // from the memory bank.
  // Then let the old generation die
  for( x = 0; x < Side; x++ )
   for( y = 0; y < Side; y++ )
    World[ x ][ y ].winter();

 Population = 0;  // Count individuals growing during this generation
 Fitness = 0;      // Sample strengths summing all individuals

 for( x = 0; x < Side; x++ )
  for( y = 0; y < Side; y++ )
   Fight( x, y );    // Let the seed struggle for growth
                     // The winner will occupy the position
 Total_generations++;
}

void Search99()  // Search a state in which the most common species has a fitness of 99% or more
{
 int displaycounter = 0,
     max_generations;

// Future extension:
// if( Genes > 1 )
//  if( Form1 -> Edit1  -> CheckBox1 == cbChecked = Gene_coupling = true;
 max_generations = Total_generations + MAX_SEARCH;
 while( Fitness < 0.99 && Total_generations < max_generations )
 {
  One_generation();
  if( displaycounter++ > 9 ) // Accuracy in determination is ten generations
  {
   Speciation();
   Draw();
   displaycounter = 0;
  }
 }
 Speciation();
 Form1 -> Edit1  -> Text = Duration;
 Form1 -> Edit4 ->  Text    = Fitness * 100;
 Form1 -> Edit5 ->  Text    = Find_favor();
 Form1 -> Edit6 ->  Text    = Total_generations;
 Form1 -> Edit7 ->  Text    = Mutations;
 Draw();
}

void Speciation()
{
 int s,
     x,
     y;

 // Clear partition array
 for( s = 0; s < 256; s++ )
 {
  Species[ s ].n = 0;
  Species[ s ].fitness = 0;
 }
 for( x = 0; x < Side; x++ )
  for( y = 0; y < Side; y++ )
  {
   if( World[ x ][ y ].plant > NONE )
   {
    s = Individual[ World[ x ][ y ].plant ].species;
    Species[ s ].n++;
    Species[ s ].fitness += Individual[ World[ x ][ y ].plant ].strength;
   }
  }
 Dominant = 0;
 for( s = 0; s < 256; s++ )
  if( Species[ s ].n > Species[ Dominant ].n ) Dominant = s;

 Fitness = Species[ Dominant ].fitness / Species[ Dominant ].n;
}

/***********************************************************
// Work in progress, not yet implemented

 // Display of evolution in 3D
 Coordinates_3D( 0, 0, 0, 100, Side * Side, 100 );
 // X-step = 1 %


************************************************************/